#!/bin/sh
#
# $Id: collect_data_kvm.sh 15 2011-04-18 12:06:42Z robert.wen@hp.com $
#
# data collection script for SPECvirt_sc2010 under KVM
#
# TODO:
# handle physical client with no vclient with a graceful way
#

OUTPUT_DIR=/tmp/data-collection
CONFIG=/opt/SPECvirt/collect_data.cfg

# temp settings
SCRIPT_REMOTE=/tmp/collect_data_kvm.sh
BASEDIR_SYS=/tmp/data-collection-sys
BASEDIR_SW=/tmp/data-collection-sw

if echo "$0" | grep -q '^\/.*'
then
    SCRIPT_LOCAL="$0"
else
    SCRIPT_LOCAL="$(pwd)/$0"
fi

sanity_check()
{
    data_collect_log "Checking hostname..." 3
    ping_host "sut"

    for client in $(gen_list $PHYCLIENT_LIST)
    do
        ping_host "phyclient${client}"
    done

    for tile in $(seq $TILE_LIST)
    do
        ping_host "client${tile}" 
        ping_host "wclient${tile}" 

        vmlist="appserver batchserver infraserver mailserver webserver"
        rest=$[$tile%4]
        if [ "$rest" -eq 1 ]; then
            vmlist="$vmlist dbserver"
        fi

        for host in $vmlist
        do
            ping_host "${host}${tile}"
        done
    done

    return 0;
}

ping_host()
{
    local host="$1"
    echo ${host}
    if ! ping -w2 -c1 ${host} &>/dev/null
    then
        data_collect_log "${host} not pingable. Please check /etc/hosts!" 2
        exit 2
    fi
}

#
# not used yet
#
setup_host_key()
{
    local CLIENT="$1"
    local PASS="$2"

    expect <<ENDEXP
spawn ssh-copy-id -i /root/.ssh/id_rsa.pub root@${CLIENT}
expect "*?assword:*"
send -- "${PASS}\r"
send -- "\r"
expect eof
ENDEXP
}

#
# this will setup the key auth
#
prepare_run_env()
{
    setup_host_key sut

    for client in $(gen_list $PHYCLIENT_LIST)
    do
        setup_host_key "phyclient${client}"
    done

    for tile in $(seq $TILE_LIST)
    do
        setup_host_key "client${tile}" 

        vmlist="appserver batchserver infraserver mailserver webserver"
        rest=$[$tile%4]
        if [ "$rest" -eq 1 ]; then
            vmlist="$vmlist dbserver"
        fi

        for host in $vmlist
        do
            setup_host_key "${host}${tile}"
        done
    done
echo "hi"
}

build_dir_tree()
{
    while read DIR
    do
        mkdir -p $DIR
    done <<ENDLINE
./VIRT
./PHYS
./PHYS/SUT
./PHYS/Clients
ENDLINE
}

data_collect_log()
{
    local text="$1"
    local class="$2"
    shift 2

    if [ "$class" = "2" ]
    then
        echo "$text" 1>&2
    elif [ "$class" = "3" ]
    then
        echo $* "$text"
    fi

    echo "$(date "+%b %d %Y %T") $text" >> $OUTPUT_DIR/data-collection.log
}

gen_list()
{
    local IFS=","
    local LIST="$1"

    for i in $LIST
    do
        if echo $i | grep -q '^[0-9]*-[0-9]*$'
        then
            seq ${i%-*} ${i#*-}
        else
            echo $i
        fi
    done
}

collect_proc_info()
{
    local procfile="$1"
    if [ ! -e "$procfile" ]
    then
        return 1
    fi

    local DIR=$(dirname $procfile)
    local FILE=$(basename $procfile)

    DIR=${DIR#/}
    mkdir -p "$DIR" &>/dev/null

    if [ $? -ne 0 ]
    then
        return 1
    fi

    cat $procfile > "$DIR/$FILE"
}

collect_sys_info()
{
# hw and device info
    lspci > lspci.txt

# os info
#    hostname > hostname.txt

# network
    ifconfig > ifconfig.txt
    route -n > route.txt
    brctl show > brctl.txt 

# storage and filesystem
    collect_proc_info "/proc/partitions"
    vgdisplay -v >vgdisplay.txt 2>/dev/null
    lvdisplay > lvdisplay.txt 2>/dev/null
    mount > mount.txt
    df > df.txt

# packages and service
    rpm -qa > rpmpkgs.txt
    chkconfig --list > chkconfig.txt

# runtime info
    ps -ef | grep qemu-kv[m] > running-vms.txt
    virsh list > virsh-list.txt
    sysctl -a > sysctl.txt

# proc info
    collect_proc_info "/proc/cpuinfo"
    collect_proc_info "/proc/devices"
    collect_proc_info "/proc/meminfo"
    collect_proc_info "/proc/modules"
    collect_proc_info "/proc/swaps"
    collect_proc_info "/proc/version"
    collect_proc_info "/proc/cmdline"

# other static configuration and logs
    find /etc/fstab \
/etc/redhat-release \
/etc/iproute2 \
/etc/hosts \
/etc/inittab \
/etc/sysctl.conf \
/etc/nsswitch.conf \
/etc/kvm \
/etc/security/limits.conf \
/etc/rc.d/init.d \
/etc/rc.d/rc3.d \
/opt/common_scripts/ostune_misc.sh	\
/var/log/dmesg \
/boot/grub/grub.conf	\
/etc/bashrc		\
/etc/exports		\
/etc/rc.d/rc.local	\
2>/dev/null | cpio -o 2>/dev/null | cpio -idm --no-absolute-filenames &>/dev/null

# clear zero size file
find . -size 0 -exec rm -f {} \;

}

collect_remote_sys_info()
{
    local host="$1"

    scp -q $SCRIPT_LOCAL $host:$SCRIPT_REMOTE
    ssh $host sh $SCRIPT_REMOTE sysinfo
    ssh $host tar -cz /tmp/data-collection-sys 2>/dev/null | tar -xz 2>/dev/null
    mv tmp/data-collection-sys/* .
    rm -fr tmp
}

collect_SUT()
{
    cd SUT
    data_collect_log "Collecting SUT info..." 3
    data_collect_log "Entering $(pwd)..."

    collect_remote_sys_info "sut"

    cd ..
}

collect_clients()
{
    cd Clients
    data_collect_log "Collecting physical clients info..." 3
    data_collect_log "Entering $(pwd)..."

    for client in $(gen_list $PHYCLIENT_LIST)
    do
        dir="phyclient${client}"
        mkdir $dir && cd $dir
        data_collect_log "Collecting $dir..." 
        collect_remote_sys_info $dir &
        cd ..
    done

    wait

    data_collect_log "Collecting virtual clients info..." 3
    for vclient in $(seq $TILE_LIST)
    do
        dir="client${vclient}"
        mkdir $dir && cd $dir
        data_collect_log "Collecting $dir..."
        collect_remote_sys_info "client${vclient}" &
        cd ..

        dir="wclient${vclient}"
        mkdir $dir && cd $dir
        data_collect_log "Collecting $dir..."
        collect_remote_sys_info "wclient${vclient}" &
        cd ..
    done

    wait

    data_collect_log "Linking virtual clients to physical clients..." 3
    for client in $(gen_list $PHYCLIENT_LIST)
    do
        dir="phyclient${client}"
        cd $dir
        for x in ${VCLIENT_LIST[$client]}
        do
            ln -s ../client${x} vclient${x}
            ln -s ../wclient${x} vwclient${x}
        done
        cd ..
    done

    cd ..
}

collect_physical_configuration()
{
    cd PHYS
    data_collect_log "Entering $(pwd)..."

    collect_SUT
    collect_clients

    cd ..
}

collect_local_software_config()
{
    local type="$1"

    case $type in

        "appserver")
            java -version 2> javaVersion.out

            curl -s --connect-timeout 10 -m 20  http://localhost:8080 | grep 'Apache Tomcat' > TomcatVersion.out
            /opt/glassfish/bin/asadmin --passwordfile /opt/SPECjAppServer2004/config/passwordfile version > GlassFishVersion.out

            pushd /opt/glassfish/glassfish/domains/
            tar cf -	\
spec2004-1/{config,autodeploy,logs}  \
2> /dev/null | tar xf - -C $BASEDIR_SW &>/dev/null

	    pushd $BASEDIR_SW/spec2004-1
	    find ./ -name "*SPECjAppServer_ItemBrowserSes_*" | xargs rm -rf - 
	    find ./ -name \"*.dat*\" | xargs rm -rf -
	    pushd $BASEDIR_SW/
	    rm -f spec2004-1/config/*com_sun_appserv_pid
	    rm -f spec2004-1/config/.*com_sun_appserv_pid

            pushd /opt/tomcat
            tar cf -	\
{conf,webapps/Emulator.war}	\
2> /dev/null | tar xf - -C $BASEDIR_SW &>/dev/null

	    pushd $BASEDIR_SW/
	    ln -s conf tomcatConf
	    echo "Directory contains: /opt/glassfish/domains/spec2004-1, /opt/tomcat/webaps/Emulator.war  and /opt/tomcat/conf" > Readme.txt

        ;;

        "dbserver")
            psql --version > psql-Version.txt
            cp -p /var/lib/pgsql/9.0/data/postgresql.conf .
	    pushd /opt/SPECjAppServer2004/
	
            tar cf -	\
schema/postgresql bin/glassfish  \
2> /dev/null | tar xf - -C $BASEDIR_SW &>/dev/null
        ;;

        "batchserver")
            java -version 2> javaVersion.out
            java -jar /opt/SPECpoll/pollme.jar -v > SPECpollVersion.out

	    mkdir cpu2006

	    cp -p /opt/SPECbatch/cpu2006-virt/{virt_sc2013_run.sh,config/cpu2006-virt.cfg} cpu2006/

            #touch emptyfile

            find 	\
2>/dev/null | cpio -o 2>/dev/null | cpio -idm --no-absolute-filenames &>/dev/null
        ;;

        "infraserver")
            sum /var/www/fcgi-bin/besim_fcgi.fcgi > sum.besim_fcgi
            ldd /var/www/fcgi-bin/besim_fcgi.fcgi > ldd.besim_fcgi

            find /etc/httpd/conf \
/etc/httpd/conf.d 	\
2>/dev/null | cpio -o 2>/dev/null | cpio -idm --no-absolute-filenames &>/dev/null
            httpd -V > etc/httpd/httpdVersion.out

        ;;

        "mailserver")
            find /etc/dovecot.conf \
/etc/dovecot \
/etc/init.d/dovecot 	\
2>/dev/null | cpio -o 2>/dev/null | cpio -idm --no-absolute-filenames &>/dev/null
            dovecot --version > etc/dovecotVersion.out
            dovecot --build-options >> etc/dovecotVersion.out
        ;;

        "webserver")
            php --info > phpinfo.out 2>/dev/null
            grep -i "^[ ]*DocumentRoot" /etc/httpd/conf/httpd.conf |tr -d '\"' |xargs lsattr -R - 2>/dev/null  |grep support |grep -v dir000 > lsattr.out

            find /etc/httpd/conf \
/etc/httpd/conf.d \
/etc/php.ini \
/var/www/html/support/init_vars.php	\
2>/dev/null | cpio -o 2>/dev/null | cpio -idm --no-absolute-filenames &>/dev/null
            httpd -V > etc/httpd/httpdVersion.out
	    cp -p /var/www/html/support/init_vars.php .
	    modinfo xnap > modinfo-xnap.out
        ;;

    esac
}

collect_remote_software_config()
{
    local host="$1"
    local type="$2"

    scp -q $SCRIPT_LOCAL $host:$SCRIPT_REMOTE
    ssh $host sh $SCRIPT_REMOTE "software" $type
    ssh $host tar -cz /tmp/data-collection-sw 2>/dev/null | tar -xz 2>/dev/null
    mv tmp/data-collection-sw/* .
    rm -fr tmp
}

collect_software_config()
{
    local type="$1"
    local tile="$2"
    local host="${type}${tile}"

    data_collect_log "Collecting software configuration on ${host}..."
    mkdir SOFTW && cd SOFTW
    collect_remote_software_config "$host" "$type" &
    cd ..
}

collect_vm_config()
{
    local host="$1"

    data_collect_log "Collecting vm configuration on ${host}..."
    mkdir VM && cd VM
    collect_remote_sys_info "$host" &
    cd ..
}


collect_one_tile()
{
    local tile="$1"
    local host=""

    vmlist="appserver batchserver infraserver mailserver webserver"
    rest=$[$tile%4]
    if [ "$rest" -eq 1 ]; then
        vmlist="$vmlist dbserver"
    fi

    for host in $vmlist
    do
        mkdir $host && cd $host
        #collect KVM xml for tile
        if [ ${tile} -lt 10 ]
	then
	    scp sut:/etc/libvirt/qemu/${host}0${tile}*.xml . 
	else
	    scp sut:/etc/libvirt/qemu/${host}${tile}*.xml .
	fi

        data_collect_log "Collecting ${host} in tile${tile}..."

        collect_software_config "$host" "$tile"
        collect_vm_config "${host}${tile}"

        cd ..
    done

    # per tile wait
    #wait
}

collect_virtual_configuration()
{
    cd VIRT
    data_collect_log "Entering $(pwd)..."

    for tile in $(seq $TILE_LIST)
    do
        mkdir "tile${tile}" && cd "tile${tile}"
        data_collect_log "Collecting tile${tile}..." 3
        data_collect_log "Entering $(pwd)..."
        collect_one_tile "${tile}"
        cd ..
    done

    cd ..
}

#
# main routine starts here
#

if [ "$1" = "sysinfo" ]
then
    rm -fr $BASEDIR_SYS 2>/dev/null && mkdir $BASEDIR_SYS 2>/dev/null && cd $BASEDIR_SYS
    collect_sys_info
elif [ "$1" = "software" ]
then
    rm -fr $BASEDIR_SW 2>/dev/null && mkdir $BASEDIR_SW 2>/dev/null && cd $BASEDIR_SW
    collect_local_software_config "$2"
elif [ "$1" = "prepare" ]
then
    prepare_run_env
else

    if [ -r "$CONFIG" ]
    then
        source $CONFIG
    else
        data_collect_log "Config $CONFIG not readable, please check!" 2
        exit 1
    fi

    rm -fr $OUTPUT_DIR 2>/dev/null && mkdir $OUTPUT_DIR 2>/dev/null && cd $OUTPUT_DIR
    sanity_check

    build_dir_tree

    date > Data-Collection-Start.txt

    collect_physical_configuration

    collect_virtual_configuration

    # final wait
    wait

    date > Data-Collection-End.txt

    data_collect_log "Data has been collected under $OUTPUT_DIR" 3

    mv data-collection.log /tmp/

    exit 0
fi

